#include <math.h>
#include <stdlib.h>
#include <gsl/gsl_siman.h>

/* $B%7%_%e%l!<%F%#%I!&%"%K!<%j%s%0$r<B9T$9$k$?$a$N%Q%i%a!<%?$N@_Dj(B */ 
/* $B<!$NC5:wE@$r7h$a$k$?$a$N<~JUC5:w$N2s?t(B */ 
#define N_TRIES 200

/* $B3F29EY$G2?2s$^$G7+$jJV$9$+(B */ 
#define ITERS_FIXED_T 1000

/* $BMp?tC5:w$N:GBgC5:w5wN%(B */ 
#define STEP_SIZE 1.0

/* $B%\%k%D%^%sDj?t(B */ 
#define K 1.0

/* $B29EY$N=i4|CM(B */ 
#define T_INITIAL 0.008

/* $B29EYDc2<78?t(B */ 
#define MU_T 1.003
#define T_MIN 2.0e-6

gsl_siman_params_t params
    = {N_TRIES, ITERS_FIXED_T, STEP_SIZE,K, T_INITIAL, MU_T, T_MIN};

/* $B0l<!85$G$N%F%9%H$N$?$a$N4X?t(B */ 
double E1(void *xp)
{
    double x = * ((double *) xp);
    return exp(-pow((x-1.0),2.0))*sin(8*x);
}

double M1(void *xp, void *yp)
{
    double x = *((double *) xp);
    double y = *((double *) yp);
    return fabs(x - y);
}

void S1(const gsl_rng * r, void *xp, double step_size)
{
    double old_x = *((double *) xp);
    double new_x;
    double u = gsl_rng_uniform(r);

    new_x = u * 2 * step_size - step_size + old_x;
    memcpy(xp, &new_x, sizeof(new_x));
}

void P1(void *xp)
{
    printf ("%12g", *((double *) xp));
}

int main(int argc, char *argv[])
{
    const gsl_rng_type * T;
    gsl_rng * r;
    double x_initial = 15.5;

    gsl_rng_env_setup();
    T = gsl_rng_default;
    r = gsl_rng_alloc(T);

    gsl_siman_solve(r, &x_initial, E1, S1, M1, P1, NULL, NULL,
                    NULL, sizeof(double), params);
    gsl_rng_free(r);
    return 0;
}
